@@ -8,7 +8,9 @@
 #include <KLocalizedString>
 #include <PortalSession.h>
 #include <QAction>
+#include <QCoreApplication>
 #include <QDBusInterface>
+#include <QMenu>
 #include <RdpConnection.h>
 #include <Server.h>
 
@@ -32,7 +34,6 @@ public:
         connect(connection->videoStream(), &KRdp::VideoStream::enabledChanged, this, &SessionWrapper::onVideoStreamEnabledChanged);
         connect(connection->videoStream(), &KRdp::VideoStream::requestedFrameRateChanged, this, &SessionWrapper::onRequestedFrameRateChanged);
         connect(connection->inputHandler(), &KRdp::InputHandler::inputEvent, session.get(), &KRdp::AbstractSession::sendEvent);
-        connect(connection, &KRdp::RdpConnection::stateChanged, this, &SessionWrapper::setSNIStatus);
         connect(connection, &QObject::destroyed, this, &SessionWrapper::onConnectionDestroyed);
     }
 
@@ -67,31 +68,6 @@ public:
         Q_EMIT connectionDestroyed(this);
     }
 
-    void setSNIStatus()
-    {
-        if (!m_sni) {
-            return;
-        }
-        if (!connection) {
-            m_sni->setStatus(KStatusNotifierItem::Passive);
-            return;
-        }
-        switch (connection->state()) {
-        case KRdp::RdpConnection::State::Closed:
-        case KRdp::RdpConnection::State::Initial:
-            m_sni->setStatus(KStatusNotifierItem::Passive);
-            break;
-        case KRdp::RdpConnection::State::Starting:
-        case KRdp::RdpConnection::State::Running:
-        case KRdp::RdpConnection::State::Streaming:
-            m_sni->setStatus(KStatusNotifierItem::Active);
-            break;
-        default:
-            m_sni->setStatus(KStatusNotifierItem::Passive);
-            break;
-        }
-    }
-
     Q_SIGNAL void sessionError();
     Q_SIGNAL void connectionDestroyed(SessionWrapper *wrapper);
 
@@ -107,12 +83,17 @@ SessionController::SessionController(KRdp::Server *server, SessionType sessionTy
     connect(m_server, &KRdp::Server::newConnectionCreated, this, &SessionController::onNewConnection);
     // Status notification item
     m_sni = new KStatusNotifierItem(u"krdpserver"_qs, this);
+    auto menu = new QMenu(u"quitMenu"_qs);
+    // Disable default quit button since it has confirmation dialog
+    m_sni->setStandardActionsEnabled(false);
     m_sni->setTitle(i18n("RDP Server"));
     m_sni->setIconByName(u"preferences-system-network-remote"_qs);
     m_sni->setStatus(KStatusNotifierItem::Passive);
-    auto quitAction = new QAction(i18n("Quit"), this);
+    auto quitAction = new QAction(i18n("Quit"), menu);
+    quitAction->setIcon(QIcon::fromTheme(QStringLiteral("application-exit")));
     connect(quitAction, &QAction::triggered, this, &SessionController::stopFromSNI);
-    m_sni->addAction(u"quitAction"_qs, quitAction);
+    menu->addAction(quitAction);
+    m_sni->setContextMenu(menu);
 
     // Create a single temporary session to request permissions from the portal
     // on startup.
@@ -177,6 +158,7 @@ void SessionController::stopFromSNI()
                         u"org.freedesktop.systemd1.Unit"_qs);
 
     unit.asyncCall(u"Stop"_qs);
+    QCoreApplication::quit();
 }
 
 std::unique_ptr<KRdp::AbstractSession> SessionController::makeSession()
